/**  
 * This is a driver program for the MolDyn simulation.  It contains
 * a constructor for the driver class MolTest# and a main.
 **/

import java.io.*;
import java.awt.*;
import java.awt.event.*;

public class MolDyn extends Frame implements Runnable{

  //Define the class data.  First, set up the particle array.
  public static final int NPART = 3; // Number of particles in the system.
  public double DNPART = NPART;
  public Particle[] system;      // system is an array of NPART particles.
  public Particle[] systemOld;
  Vect force[] = new Vect[NPART];
  double energy,time;

  //Now define the system dimensions and the time step.
  public static final double XDIM = 10.;
  public static final double YDIM = 10.;
  static double DELT = 0.005;

  //These will be used to delay the execution of the thread.
  long startTime,endTime;
  public static final int DELAY=10;

  // Create a canvas object to display the particles.
  MolCanvas canvas;

  // Run the program as a thread, since the calculations are
  // done much more quickly than the graphics.
  Thread MolThread=null;

  // This is the constructor.  Here we allocate the arrays for the system
  // and for the old positions, creat the canvas, and initialize the system.
  public MolDyn() {
    super("Molecular Dynamics Simulation");
    system = new Particle[NPART];   
    systemOld = new Particle[NPART]; 
    add(canvas=new MolCanvas(system));
    init();
    
    if(MolThread==null) {
	MolThread=new Thread(this);
  	MolThread.start();
	MolThread.setPriority(Thread.MIN_PRIORITY);
    }                       
  }

  // init() is a method for assigning particle positions.
  // The old and new configurations are initially both the same.
  public void init() {
    double x,y;
    double xold,yold;
    double vx[],vy[];                // Velocity components of a particle.
    double fs = 0.0;
    double temp = 0.5;            // Initial (dummy) temperature value;
    double sumvx=0.0; double sumvy=0.0;
    double sumv2=0.0;
    vx = new double[NPART];
    vy = new double[NPART];

    x = 0.0; y = 0.0;       // Set initial positions and velocities.
    double step = 1.;
    Vect r,rold,v;           // Declare two temp. vectors for initializing.

    for (int i = 0; i < NPART; i++) {
      vx[i] = (Math.random()-0.5);
      vy[i] = (Math.random()-0.5);
      sumvx += vx[i]; sumvy += vy[i];
      sumv2 += vx[i]*vx[i] + vy[i]*vy[i];
    }
    sumvx = sumvx/DNPART;
    sumvy = sumvy/DNPART;
    sumv2 = sumv2/DNPART;
    fs = Math.sqrt(3.*temp/sumv2);

    for (int i = 0; i < NPART; i++) {
      vx[i] = (vx[i] - sumvx)*fs;
      vy[i] = (vy[i] - sumvy)*fs;
      r = new Vect(x,y);          // Vect r gets assigned some coordinates.
      v = new Vect(vx[i],vy[i]);      // Vect v gets assigned some velocities. 
      system[i] = new Particle(r,v);  // The ith particle gets position r and
                                      // velocity v.
      xold = x - vx[i]*DELT;
      yold = y - vy[i]*DELT;
      rold = new Vect(xold,yold);
      systemOld[i] = new Particle(rold,v);

      x += step;                      // Update positions and velocities.
      if(x >= XDIM){
	x = 0.0;
	y += step;
      }
    }
  }

  //Since we are using Threads, we must make the class runnable and 
  // override the run() method.
  public void run() {
	Thread myThread=Thread.currentThread();
	while(MolThread==myThread){
		startTime=System.currentTimeMillis();
		calcForce();
		integrate();
		canvas.repaint();
		time+=DELT;

		try{
			Thread.sleep(DELAY);
//			endTime=System.currentTimeMillis();
//			if((DELAY-(endTime-startTime))>0) {
//				Thread.sleep(DELAY-(endTime-startTime));
//			} else
//				Thread.sleep(DELAY);
		}
		catch(InterruptedException e) {}
	}			
  }

  // writeOutput() prints out the particle positions.
  public void writeOutput() throws IOException {
    File outputFile1 = new File("positions.dat");
    File outputFile2 = new File("velocities.dat");
    
    FileWriter out1 = new FileWriter(outputFile1);
    FileWriter out2 = new FileWriter(outputFile2);

    for (int i=0;i<NPART;i++) {
      out1.write(system[i].pos.x + " " + system[i].pos.y + "\n");
      out2.write(system[i].vel.x + " " + system[i].vel.y + "\n");
    }
    out1.close();
    out2.close();
  } 

  // Calculates the forces.  Uses a 6-12 Lennard-Jones interaction.
  // May want to rewrite the program so that
  // the vector addition is hidden in the Vect class.
  public void calcForce() {
    double r2mag,ff,r6magi,r2magi;
    Vect rdiff = new Vect();
    energy = 0.0;                           // Zero the energy.
    for (int i = 0; i < NPART; i++) {
      force[i] = new Vect();              // Initialize forces.
    }
    
    // This stuff should really be in a Vect method and hidden at this
    // level.
    for (int i = 0; i < NPART; i++) {
      for (int j = i+1; j < NPART; j++) {
	rdiff.x = system[i].pos.x - system[j].pos.x;  // Vector differences.
	if(rdiff.x > XDIM/2.) rdiff.x-=XDIM;
	if(rdiff.x < -XDIM/2.) rdiff.x+=XDIM;
	
	rdiff.y = system[i].pos.y - system[j].pos.y;
	if(rdiff.y > YDIM/2.) rdiff.y-=YDIM;
	if(rdiff.y < -YDIM/2.) rdiff.y+=YDIM;

	r2mag = rdiff.x * rdiff.x + rdiff.y * rdiff.y; 
	r2magi = 1./r2mag;
	r6magi = r2magi * r2magi * r2magi;
	ff = 48. * r2magi * r6magi * (r6magi - 0.5);

	force[i].x = force[i].x + ff*rdiff.x;      // Force on atom i by j.
	force[i].y = force[i].y + ff*rdiff.y;	  

	force[j].x = force[j].x - ff*rdiff.x;      // Force on j by i.
	force[j].y = force[j].y - ff*rdiff.y;	  
	
	energy += 4. * r6magi * (r6magi - 1.); 
      }
    }
  }

  // Integrates the equations of motion.
  public void integrate() {
    Vect sumv = new Vect();      // Holds summed velocity components.
    Vect rnew = new Vect();      // Temporarily store new positions.
    Vect vtmp = new Vect();
    double sumv2 = 0.0;
    double temp = 0.0;
    double etot = 0.0;

    // Should write a method verlet() that does this stuff.
    for (int i = 0; i <  NPART; i++) {
      rnew.x = 2.0 * system[i].pos.x - 
	systemOld[i].pos.x + DELT * DELT * force[i].x;
      if(rnew.x > XDIM) rnew.x-=XDIM;
      if(rnew.x < 0.0) rnew.x+=XDIM;
      
      rnew.y = 2.0 * system[i].pos.y - 
	systemOld[i].pos.y + DELT * DELT * force[i].y;
      if(rnew.y > YDIM) rnew.y-=YDIM;
      if(rnew.y < 0.0) rnew.y+=YDIM;
      vtmp.x = (rnew.x - systemOld[i].pos.x) / (2. * DELT);
      vtmp.y = (rnew.y - systemOld[i].pos.y) / (2. * DELT);
      sumv.x += vtmp.x;
      sumv.y += vtmp.y;
      sumv2 += vtmp.x * vtmp.x + vtmp.y * vtmp.y;
      systemOld[i].pos.x = system[i].pos.x;
      systemOld[i].pos.y = system[i].pos.y;
      system[i].pos.x = rnew.x;
      system[i].pos.y = rnew.y;
    }
    temp = sumv2 / (3. * DNPART);
    etot = energy + sumv2 / (2. * DNPART);
  }
  
  // Finally, here's the main() for the program.
  public static void main(String[] args) throws IOException {
    int niter=10;
    double time = 0.0;
    MolDyn m = new MolDyn();

    //Set up the window, make it closable, and show it.
    m.setSize(400,400);
    m.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {System.exit(0);}
    });
    m.show();
    m.run();
    m.writeOutput();
  }
}

/***********************************************************************
 * A canvas class to draw the figure.  Draws circles for the partilces.
 * The radius of the particles is the LJ parameter epsilon, but I'm
 * working in units where eps=1.
 **********************************************************************/

class MolCanvas extends Canvas {

   // Some variables.
   double RADIUS=0.5;  
   int w,h,ScaleX,ScaleY,ScaleR;
   Particle[] system;    

   // These are needed for double buffering.
   Dimension offDimension;
   Image offImage;
   Graphics offGraphics;

   //A constructor.
   public MolCanvas(Particle[] system) {
	this.system=system;
   }

   //Override the paint() method.
   public void paint(Graphics g) {
	update(g);
   }

   public void update(Graphics g) {
	Dimension d=getSize();
	//Create the offscreen image.
	if((offGraphics==null)
		||(d.width!=offDimension.width)
		||(d.height!=offDimension.height)) {
			offDimension= d;
			offImage=createImage(d.width,d.height);
			offGraphics=offImage.getGraphics();
		}
	//Erase the previous image.
	offGraphics.setColor(getBackground());
	offGraphics.fillRect(0,0,d.width,d.height);
	offGraphics.setColor(Color.black);	

	w=getSize().width;
   	h=getSize().height;
   	ScaleR=(int)(RADIUS/MolDyn.XDIM*w);

	//Draw filled in circles and put a ring around them.
	for(int i=0;i<MolDyn.NPART;i++){
		ScaleX=(int)(system[i].pos.x/MolDyn.XDIM*w);
		ScaleY=(int)(system[i].pos.y/MolDyn.YDIM*h);
 		offGraphics.setColor(Color.red);
		offGraphics.fillOval(ScaleX-ScaleR,ScaleY-ScaleR,2*ScaleR,2*ScaleR);
		if((ScaleX-ScaleR)<0) 
	          offGraphics.fillOval(ScaleX+w-ScaleR,ScaleY-ScaleR,2*ScaleR,2*ScaleR);
		if((ScaleX+ScaleR)>w) 
	          offGraphics.fillOval(ScaleX-w-ScaleR,ScaleY-ScaleR,2*ScaleR,2*ScaleR);
		if((ScaleY-ScaleR)<0) 
	          offGraphics.fillOval(ScaleX-ScaleR,ScaleY+h-ScaleR,2*ScaleR,2*ScaleR);
		if((ScaleY+ScaleR)>h) 
	          offGraphics.fillOval(ScaleX-ScaleR,ScaleY-h-ScaleR,2*ScaleR,2*ScaleR);

		offGraphics.setColor(Color.black);
		offGraphics.drawOval(ScaleX-ScaleR,ScaleY-ScaleR,2*ScaleR,2*ScaleR);
		if((ScaleX-ScaleR)<0) 
	          offGraphics.drawOval(ScaleX+w-ScaleR,ScaleY-ScaleR,2*ScaleR,2*ScaleR);
		if((ScaleX+ScaleR)>w) 
	          offGraphics.drawOval(ScaleX-w-ScaleR,ScaleY-ScaleR,2*ScaleR,2*ScaleR);
		if((ScaleY-ScaleR)<0) 
	          offGraphics.drawOval(ScaleX-ScaleR,ScaleY+h-ScaleR,2*ScaleR,2*ScaleR);
		if((ScaleY+ScaleR)>h) 
	          offGraphics.drawOval(ScaleX-ScaleR,ScaleY-h-ScaleR,2*ScaleR,2*ScaleR);

	}
	g.drawImage(offImage,0,0,this);
   } 
}
